package net.sjava.cassandra.manager;

import java.nio.ByteBuffer;
import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import net.sjava.cassandra.model.Comment;
import net.sjava.cassandra.util.ColumnUtil;
import net.sjava.cassandra.util.ConnectionUtil;
import net.sjava.cassandra.util.ConverterUtil;

import org.apache.cassandra.thrift.Cassandra;
import org.apache.cassandra.thrift.Column;
import org.apache.cassandra.thrift.ColumnOrSuperColumn;
import org.apache.cassandra.thrift.ColumnParent;
import org.apache.cassandra.thrift.ColumnPath;
import org.apache.cassandra.thrift.ConsistencyLevel;
import org.apache.cassandra.thrift.IndexClause;
import org.apache.cassandra.thrift.IndexExpression;
import org.apache.cassandra.thrift.IndexOperator;
import org.apache.cassandra.thrift.KeySlice;
import org.apache.cassandra.thrift.SlicePredicate;
//import org.apache.log4j.Logger;

//import net.sjava.cassandra.util.ConverterUtil;

/**
 *  ϴ Ŵ Ŭ 
 * 
 * @author mcsong@gmail.com
 *
 */
public class CommentManager {

	//private static final Logger LOG = Logger.getLogger(CommentManager.class);
	private static String UTF8 ="UTF8";
	
	private Cassandra.Client client;
	private String cfName = "comments";
	private ColumnParent cp = new ColumnParent(cfName);
		
	// ̱ ü 
	public static CommentManager instance = new CommentManager();
	
	private CommentManager() { 
		try {
			client  = ConnectionUtil.createConnection("BLOG");
		} catch(Exception e) {
			e.printStackTrace();
		}
	}
	
	public static CommentManager instance() {
		return  instance;
	}
	
	/**
	 *  ԷѴ. 
	 * 
	 * @param key
	 * @param postId
	 * @param comment
	 * @throws Exception
	 */
	public void addComment(long key, String postId, String comment) throws Exception {
		//LOG.debug(String.format("addComment(%s, %s) start", postId, comment));
		
		long timestamp = System.currentTimeMillis();
		
		Column idColumn = ColumnUtil.createColumn("post_id".getBytes(UTF8), postId.getBytes(UTF8), timestamp);
    	Column commentColumn = ColumnUtil.createColumn("comment".getBytes(UTF8), comment.getBytes(UTF8), timestamp);
    	Column dateColumn = ColumnUtil.createColumn("comment_date".getBytes(UTF8), timestamp, timestamp);
    	
    	ByteBuffer rowKey = ConverterUtil.longToBytebuffer(key);
    	
    	client.insert(rowKey, cp, idColumn, ConsistencyLevel.ONE);
    	client.insert(rowKey, cp, commentColumn, ConsistencyLevel.ONE);
    	client.insert(rowKey, cp, dateColumn, ConsistencyLevel.ONE);
    	
		//LOG.debug(String.format("addComment(%s, %s) end", postId, comment));
	}	
	
	/**
	 *  ۿ   Ʈ ´. 
	 * 
	 * @param postID
	 * @return
	 * @throws Exception
	 */
	public List<Comment> getComments(String postID) throws Exception {
		// 
		IndexClause indexClause = new IndexClause();
		indexClause.start_key = ByteBuffer.allocate(0);
		IndexExpression indexExpression = new IndexExpression();
		indexExpression.column_name = ByteBuffer.wrap("post_id".getBytes("UTF-8"));
		indexExpression.value = ByteBuffer.wrap(postID.getBytes("UTF-8"));
		indexExpression.op = IndexOperator.EQ;
		
		indexClause.addToExpressions(indexExpression);
		
		SlicePredicate predicate = new SlicePredicate();
		predicate.addToColumn_names(ByteBuffer.wrap("post_id".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("comment".getBytes("UTF8")));
		predicate.addToColumn_names(ByteBuffer.wrap("comment_date".getBytes("UTF8")));
				
		List<Comment> comments = new ArrayList<Comment>();
		
		// Ű ̽ ȯȴ
		List<KeySlice> keys = client.get_indexed_slices(cp, indexClause, predicate, ConsistencyLevel.ONE);
		
		// Ű  ֽ Ű   ´. 
		Collections.reverse(keys);
		
		for (KeySlice ks : keys) {
			List<ColumnOrSuperColumn> coscs = ks.columns;
			
			// ÷ ̸  ȮѴ. 
			String name = null;
			Comment comment = Comment.create();
			comment.setKey(ByteBuffer.wrap(ks.getKey()).getLong());
			for (ColumnOrSuperColumn cs : coscs) {
				name = ConverterUtil.byteBufferToString(cs.column.name);

				if("post_id".equals(name)) 
					comment.setId(ConverterUtil.byteBufferToString(cs.column.value));
				
				if("comment".equals(name))
					comment.setComment(ConverterUtil.byteBufferToString(cs.column.value));
				
				if("comment_date".equals(name))
					comment.setDate(cs.column.value.getLong());
			}
			
			comments.add(comment);
		}
		
		return comments;
	}
	
	/**
	 *  Ѵ. 
	 * 
	 * @param key
	 * @throws Exception
	 */
	public void deleteComment(long key) throws Exception {
		//LOG.debug(String.format("deleteComment(%d) start", key));
				
		long timestamp = System.currentTimeMillis();
		
		ColumnPath cPath = new ColumnPath();
		cPath.setColumn_family(cfName);
		
		client.remove(ConverterUtil.longToBytebuffer(key) , cPath, timestamp, ConsistencyLevel.ONE);
		//LOG.debug(String.format("deleteComment(%d) end", key));
	}
	
}
